@@ -116,9 +116,7 @@ class AbstractBaseUser(models.Model):
 
     def has_usable_password(self):
         """
-        Return False if set_unusable_password() has been called for this user,
-        or if the password is None, or if the password uses a hasher that's not
-        in the PASSWORD_HASHERS setting.
+        Return False if set_unusable_password() has been called for this user.
         """
         return is_password_usable(self.password)
 
@@ -138,4 +136,4 @@ class AbstractBaseUser(models.Model):
 
     @classmethod
     def normalize_username(cls, username):
-        return unicodedata.normalize('NFKC', username) if username else username
+        return unicodedata.normalize('NFKC', username) if isinstance(username, str) else username
